//+------------------------------------------------------------------+
//|                                                     Parabolic    |
//|                           Chris Hinton  - chrisphinton@gmail.com |
//|                                                                  |
//+------------------------------------------------------------------+
/*


*/

#property copyright "Chris Hinton"

// Shift of moving average
//#define Shift 1

// Trend Detection function
#define BULL 111111
#define BEAR 222222

// Input variables
extern string  comment2          = "*** Order Options ***";
extern double  Lots              = 1;
extern int     Slippage          = 5;
extern string  Order_Comment     = "Chris Parabolics EURUSD";
extern int     Magic             = 123455;
extern color   Order_Arrow_Color = Green;

extern string  comment3          = "*** Moving Average Options ***";
extern double  Step      = 0.02;
extern double  Maximum   = 0.2;
extern int     Long_MA_Applied_Price = 4;

extern int     Short_MA_Period   = 50;

extern int     Long_MA_Period   = 200;
extern int     Short_MA_Method    = 0;
extern int     Short_MA_Applied_Price = 4;

// Global variables
int Total_Open_Orders = 1; // we only want to open one order at a time and only manage the one order
int cnt = 0;               // counter variable, used in for() loops
bool init_variables;       // init variable when program starts
datetime PreviousBar;      // record the candle/bar time
int newminute;


int init()
  {
   init_variables = true;  // Allows us to init variables in the start function because 
                           // we cannot do this in the init function because the variables 
                           // use values from the chart 
   return(0);
  }
int deinit()
  {
   // clear chart when EA is removed
   ObjectsDeleteAll();
   return(0);
  }

int start()
  {
      
   // moving average only run expert advisor if there is enough candle/bars in history
   if(Bars < Long_MA_Period+1)
   {
      if(IsTesting()==false)
      {
         Comment("Long moving average does not have enough Bars in history to open a trade!\n", 
               "Must be at least ",Long_MA_Period, " bars to perform technical analysis.");
      }
      else
      {
         Comment("Back testing currently Long moving average does not have enough Bars to open a trade.\n", 
               "Must be at least ",Long_MA_Period, " bars to perform technical analysis.\n",
               "Please be patient and wait...\n",
               "Bars in history ",Bars);
      }
      return(0);
   }
            
   // make sure trader has set Lots to at least the minimum lot size of the broker and 
   // we will normalize the Lots variable so we can properly open an order
   if(MarketInfo(Symbol(),MODE_MINLOT) == 0.01)
   {
      Lots = NormalizeDouble(Lots,2);
      if(Lots < 0.01)
      {
         Comment("The variable Lots must be 0.01 or greater to open an order. ");
         return(0);
      }
   }
   if(MarketInfo(Symbol(),MODE_MINLOT) == 0.1)
   {
      Lots = NormalizeDouble(Lots,1);
      if(Lots < 0.1)
      {
         Comment("The variable Lots must be 0.1 or greater to open an order. ");
         return(0);
      }
   }
   if(MarketInfo(Symbol(),MODE_MINLOT) == 1)
   {
      Lots = NormalizeDouble(Lots,0);
      if(Lots < 1)
      {
         Comment("The variable Lots must be 1 or greater to open an order. ");
         return(0);
      }
   }
         
   // init variables when the expert advisor first starts running
   if(init_variables == true)
   {
      PreviousBar = Time[0];     // record the current canle/bar open time
      newminute = Minute();
      // place code here that you only wnat to run one time
       
      init_variables = false;    // change to false so we only init variable once
   }
   
   // perform analysis and open orders on new minute 
   if(newminute != Minute())
   {
      newminute = Minute();
      // only perform analysis and close order if we only have one order open
      if(TotalOpenOrders() == Total_Open_Orders && SelectTheOrder() == true)
      {
         if(OrderType() == OP_BUY && TrendDetection() == BEAR)
         {
            OrderClose(OrderTicket(),OrderLots(),Bid,Slippage,Order_Arrow_Color);
         }
         if(OrderType() == OP_SELL && TrendDetection() == BULL)
         {
            OrderClose(OrderTicket(),OrderLots(),Ask,Slippage,Order_Arrow_Color);
         }
      }
      
      // only perform analysis and open new order if we have not reached our Total_Open_Orders max
      if(TotalOpenOrders() < Total_Open_Orders)
      {
         // open buy
         if(TrendDetection() == BULL)
         {
            // open order
            OrderSend(Symbol(),OP_BUY,Lots,Ask,Slippage,0,0,Order_Comment,Magic,0,Order_Arrow_Color);
         }
         
         // open sell
         if(TrendDetection() == BEAR)
         {
            // open order 
            OrderSend(Symbol(),OP_SELL,Lots,Bid,Slippage,0,0,Order_Comment,Magic,0,Order_Arrow_Color);
         }
      }
      
      // when back testing only display chart info every candle/bar so we do not slow down back tests
      if(IsTesting() == true)
      {
         Display_Info();
      }
   }
      
   // when not back testing display chart info every tick
   if(IsTesting() == false)
   {
      Display_Info();
   }
   
   return(0);
  }

/////////////////////////////////////////////////////////////////////////
// Common functions                                                   //
///////////////////////////////////////////////////////////////////////

// This function returns the total amount of orders the expert advisor has open  
int TotalOpenOrders()
{
   cnt=OrdersTotal();
   int TotalOpenOrders = 0;
   
   if(cnt==0)
   {
      return(0);
   }
   else
   {
      for(;cnt>=0;cnt--)
      {
         RefreshRates();
         OrderSelect(cnt,SELECT_BY_POS);
         if(OrderMagicNumber()==Magic)
         {
            TotalOpenOrders++;
         }
      }
   }
   return(TotalOpenOrders);
}

// This function finds the open order and selects it 
int SelectTheOrder()
{
   cnt=OrdersTotal();
      
   if(cnt==0)
   {
      return(false);
   }
   else
   {
      for(;cnt>=0;cnt--)
      {
         RefreshRates();
         OrderSelect(cnt,SELECT_BY_POS);
         if(OrderMagicNumber()==Magic)
         {
            return(true);
         }
      }
   }
   return(false);
}

/*// This function return the value true if the current bar/candle was just formed
bool NewBar()
{
   if(PreviousBar<Time[0])
   {
      PreviousBar = Time[0];
      return(true);
   }
   else
   {
      return(false);
   }
   return(false);    // in case if - else statement is not executed
}*/

// is trend up/bullish or is it down/bearish
int TrendDetection()
{
   // BULL trend
   //MessageBox("Test done. Bid = " + Bid + "  Parabolic = " + iSAR(NULL,0,Step,Maximum,0));
   
   if (iSAR(NULL,0,Step,Maximum,0) < Bid)
   {
      return(BULL);
   }
   
   // BEAR trend
   if (iSAR(NULL,0,Step,Maximum,0) > Ask)
   {
      return(BEAR);
   }
      
   // flat no trend return 0
   return(0);
}

void Display_Info()
{
   Comment("Firesuite Parabolic v1.0\n",
            "Forex Account Server:",AccountServer(),"\n",
            "Account Balance:  $",AccountBalance(),"\n",
            "Lots:  ",Lots,"\n",
            "Symbol: ", Symbol(),"\n",
            "Price:  ",NormalizeDouble(Bid,4),"\n",
            "Pip Spread:  ",MarketInfo("EURUSD",MODE_SPREAD),"\n",
            "Date: ",Month(),"-",Day(),"-",Year()," Server Time: ",Hour(),":",Minute(),":",Seconds(),"\n",
            "Minimum Lot Size: ",MarketInfo(Symbol(),MODE_MINLOT));
   return(0);
}